import textwrap
from pygame import Rect
from pygame.locals import *
from widget import Widget
from controls import Label, TextField, Button, Row, Column

class Dialog(Widget):

	bg_color = (128, 128, 128)
	enter_response = True
	cancel_response = False

	def key_down(self, event):
		k = event.key
		if k == K_RETURN or k == K_KP_ENTER:
			if self.enter_response is not None:
				self.dismiss(self.enter_response)
		elif k == K_ESCAPE:
			if self.cancel_response is not None:
				self.dismiss(self.cancel_response)

def wrapped_label(text, wrap_width):
	paras = text.split("\n\n")
	text = "\n".join([textwrap.fill(para, wrap_width) for para in paras])
	return Label(text)

def alert(mess, wrap_width = 60, bg_color = None):
	lb = wrapped_label(mess, wrap_width)
	lb.rect.topleft = (10, 10)
	box = Dialog(Rect((0, 0), lb.rect.bottomright))
	box.rect.inflate_ip((20, 20))
	box.add(lb)
	if bg_color:
		box.bg_color = bg_color
	return box.present()

def ask(mess, responses = ["OK", "Cancel"], default = 0, cancel = -1,
		wrap_width = 60, bg_color = None):
	lb = wrapped_label(mess, wrap_width)
	lb.rect.topleft = (10, 10)
	buts = []
	for caption in responses:
		but = Button(caption, action = lambda x = caption: box.dismiss(x))
		buts.append(but)
	brow = Row(buts, spacing = 20)
	col = Column([lb, brow], spacing = 10, align ='r')
	col.rect.topleft = (10, 10)
	box = Dialog()
	if default is not None:
		box.enter_response = responses[default]
	else:
		box.enter_response = None
	if cancel is not None:
		box.cancel_response = responses[cancel]
	else:
		box.cancel_response = None
	box.border_width = 1
	if bg_color:
		box.bg_color = bg_color
	box.add(col)
	box.shrink_wrap()
	return box.present()

def input_text(prompt, width, initial = None):
	def ok():
		box.dismiss(True)
	def cancel():
		box.dismiss(False)
	lb = Label(prompt)
	tf = TextField(width)
	if initial:
		tf.set_text(initial)
	tf.enter_action = ok
	tf.escape_action = cancel
	tf.rect.top = lb.rect.top
	tf.rect.left = lb.rect.right + 5
	box = Widget(Rect((0, 0), tf.rect.bottomright))
	box.add(lb)
	box.add(tf)
	box.bg_color = (128, 128, 128)
	tf.focus()
	if box.present():
		return tf.get_text()
	else:
		return None
