import sys
import pygame
from pygame.locals import *
from pygame.time import get_ticks
import widget
from widget import Widget

mod_cmd = KMOD_LCTRL | KMOD_RCTRL | KMOD_LMETA | KMOD_RMETA
double_click_time = 300 # milliseconds

modifiers = dict(
	shift = False,
	ctrl = False,
	alt = False,
	meta = False,
)

modkeys = {
	K_LSHIFT: 'shift',  K_RSHIFT: 'shift',
	K_LCTRL:  'ctrl',   K_RCTRL:  'ctrl',
	K_LALT:   'alt',    K_RALT:   'alt',
	K_LMETA:  'meta',   K_RMETA:  'meta',
}

def set_modifier(key, value):
	attr = modkeys.get(key)
	if attr:
		modifiers[attr] = value

def add_modifiers(event):
	d = event.dict
	d.update(modifiers)
	d['cmd'] = event.ctrl or event.meta

class RootWidget(Widget):

	fg_color = (255, 255, 255)
	bg_color = (0, 0, 0)
	shift = False
	redraw_every_frame = True
	do_draw = False

	def __init__(self, surface):
		Widget.__init__(self, surface.get_rect())
		self.surface = surface
		widget.root_widget = self
	
	def set_timer(self, ms):
		pygame.time.set_timer(USEREVENT, ms)

	def run(self, modal_widget = None):
		modal_widget = modal_widget or self
		was_modal = modal_widget.is_modal
		modal_widget.is_modal = True
		modal_widget.modal_result = None
		mouse_widget = None
		num_clicks = 0
		last_click_time = 0
		self.do_draw = True
		while modal_widget.modal_result is None:
			if self.do_draw:
				self.draw_all(self.surface)
				self.do_draw = False
			pygame.display.flip()
			events = [pygame.event.wait()]
			events.extend(pygame.event.get())
			for event in events:
				type = event.type
				if type == QUIT:
					self.quit()
				elif type == MOUSEBUTTONDOWN:
					self.do_draw = True
					t = get_ticks()
					if t - last_click_time <= double_click_time:
						num_clicks += 1
					else:
						num_clicks = 1
					last_click_time = t
					event.dict['num_clicks'] = num_clicks
					event.dict['shift'] = self.shift
					add_modifiers(event)
					event_widget = self.find_widget(event.pos)
					if event_widget.is_inside(modal_widget):
						mouse_widget = event_widget
						event_widget.handle_mouse('mouse_down', event)
				elif type == MOUSEMOTION:
					add_modifiers(event)
					if mouse_widget:
						mouse_widget.handle_mouse('mouse_drag', event)
					else:
						event_widget = self.find_widget(event.pos)
						if event_widget.is_inside(modal_widget):
							event_widget.handle_mouse('mouse_move', event)
				elif type == MOUSEBUTTONUP:
					add_modifiers(event)
					self.do_draw = True
					if mouse_widget:
						mouse_widget.handle_mouse('mouse_up', event)
						mouse_widget = None
				elif type == KEYDOWN:
					key = event.key
					set_modifier(key, True)
					self.do_draw = True
					self.send_key(modal_widget, 'key_down', event)
				elif type == KEYUP:
					key = event.key
					set_modifier(key, False)
					self.do_draw = True
					self.send_key(modal_widget, 'key_up', event)
				elif type == USEREVENT:
					self.do_draw = self.redraw_every_frame
					self.begin_frame()
		modal_widget.is_modal = was_modal
	
	def send_key(self, widget, name, event):
		add_modifiers(event)
		widget.dispatch_key(name, event)
	
	def begin_frame(self):
		pass

	def get_root(self):
		return self

	def has_focus(self):
		return True

	def quit(self):
		sys.exit(0)
